(* Bantam -- a lightweight file manager for X11
   Copyright  2005 by Matthew C. Gushee

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*)

open Common
open Util
open Rascl
open GdkKeysyms


let config_dir =
  let user_dir =
    try Sys.getenv "BANTAM_USER_DIR"
    with Not_found -> failwith missing_vars_warning in
  Filename.concat user_dir "config"

let keymap () =
  let kmf = Filename.concat config_dir "keymap" in
  let kmd = read_config_file kmf in
  let elts = Dict.Unsafe.alist_of_dict kmd in
  let getspec (ks, cmd) =
    match ks.[0] with
    | '^' ->
      (keysym_of_string (String.sub ks 1 (String.length ks - 1)), true), cmd
    | _ -> (keysym_of_string ks, false), cmd in
  List.map getspec elts

let handlers file =
  let hd = read_config_file file in
  let type_exists t = Dict.Unsafe.exists (fun k _ -> k = t) hd in
  (* FIXME: Need a way to process unanticipated types. *)
  let major_types =
    List.filter type_exists
    ["text"; "image"; "audio"; "video"; "application"; "*"] in
  let sublists =
    List.map
      ( fun maj -> maj, Dict.Unsafe.alist_of_dict ~prefix:[maj] hd )
      major_types in
  let resolve_type major minor =
    match major, minor with
    | "text", "*" -> OtherText
    | "text", m -> Text m
    | "image", "*" -> OtherImage
    | "image", m -> Image m
    | "audio", "*" -> OtherAudio
    | "audio", m -> Audio m
    | "video", "*" -> OtherVideo
    | "video", m -> Video m
    | "application", "*" -> OtherApplication
    | "application", m -> Application m
    | "*", "*" -> Other
    | maj, min -> Custom (maj, min)
  and read_handler hspec =
    match hspec.[0] with
    | '@' ->
      ( let hname = String.sub hspec 1 (String.length hspec - 1) in
        match hname with
        | "INTERNAL" -> IntHandler
        | _ -> NoHandler )
    | _ -> ExtHandler hspec in
  let raw_hmaps =
    List.map
      ( fun (major, sublist) ->
        List.map
          ( fun (minor, hspec) -> resolve_type major minor, read_handler hspec )
          sublist )
      sublists in
  List.flatten raw_hmaps


let viewers () = handlers (Filename.concat config_dir "viewers")
let editors () = handlers (Filename.concat config_dir "editors")

let get_key_cmd (ks, shift) map =
  if shift then
    try List.assoc (ks, shift) map
    with Not_found -> List.assoc (ks, false) map
  else
    List.assoc (ks, false) map

(*
let readprefs () =
  let prf = Filename.concat config_dir "prefs" in
  let ic = open_in prf in
  let rec readlines lno =
    let line =
      try Some (input_line ic)
      with End_of_file -> None in
    match line with
    | None -> ()
    | Some ln when Str.string_match ignoreRE ln 0 -> readlines (lno+1)
    | Some ln when Str.string_match prefsRE ln 0 ->
      let name = Str.matched_group 1 ln
      and value = Str.matched_group 2 ln in
      Hashtbl.replace prefs name value;
      readlines (lno+1)
    | Some ln ->
      prerr_endline (Printf.sprintf "Possible error in line %d of %s" lno prf);
      readlines (lno+1) in
  readlines 0

let getpref name = Hashtbl.find prefs name

let dumpprefs () =
  prerr_endline "PREFERENCES:";
  Hashtbl.iter
    ( fun k v -> prerr_endline (Printf.sprintf "%s : %s" k v))
    prefs;
  flush stderr
*)
